package com.revolsys.ui.html.serializer.key;

import java.math.BigDecimal;

import com.revolsys.io.xml.XmlWriter;
import com.revolsys.util.JavaBeanUtil;
import com.revolsys.util.MathUtil;

/**
 * Serialize a percent with the % sign.
 * 
 * @author Paul Austin
 */
public class PercentKeySerializer extends AbstractKeySerializer {
  /** The scale to display the percent as. */
  private int scale = MathUtil.PERCENT_SCALE - 2;

  public PercentKeySerializer() {
  }

  public PercentKeySerializer(final String name) {
    super(name);
  }

  /**
   * Construct a new PercentKeySerializer.
   */
  public PercentKeySerializer(final String name, final int displayScale) {
    super(name);
    this.scale = displayScale;
  }

  /**
   * Serialize the value to the XML writer.
   * 
   * @param out The XML writer to serialize to.
   * @param object The object to get the value from.
   */
  public void serialize(final XmlWriter out, final Object object) {
    final BigDecimal value = JavaBeanUtil.getProperty(object, getName());
    if (value != null) {
      out.text(MathUtil.percentToString(value, scale));
    } else {
      out.text("-");
    }
  }
}
